# frozen_string_literal: true

# Karafka should be able to just consume when throttling limits are not reached.
# DLQ should have nothing to do with this.

setup_karafka

class Consumer < Karafka::BaseConsumer
  def consume
    messages.each do |message|
      DT[message.metadata.partition] << message.raw_payload
    end
  end
end

draw_routes do
  topic DT.topic do
    consumer Consumer
    dead_letter_queue(topic: DT.topics[1], max_retries: 1)
    throttling(
      limit: 1_000,
      interval: 60_000
    )
  end
end

# Should not happen
Karafka.monitor.subscribe 'filtering.throttled' do
  raise
end

elements = DT.uuids(100)
produce_many(DT.topic, elements)

start_karafka_and_wait_until do
  DT[0].size >= 100
end

assert_equal elements, DT[0]
assert_equal 1, DT.data.size
